--Globals
--[[Hello there curious viewer.  If your an experienced modder then most of these comments will be fairly obvious.
But if your just starting out and wanted to use my mod as an example for creating your own then good for you.

I built this mod with the intent to give to someone else to publish but in the end I decided it would be better
if I just published and maintained it over the workshop.  As a result alot of the scripts are commented to aid
viewers in understanding what exactly is going on.  The comments basically explain things as pseudo-code so you can
effectively understand what's going on, even if you haven't programmed in LUA in the past.

I would recommend starting in the charliethegardener.lua in the prefab folder first, as that is where
the basic adjustments for character mods are made.  Come back here when that file instructs you and I'll see you below.
--]]
local Recipe = GLOBAL.Recipe
local RECIPETABS = GLOBAL.RECIPETABS
local TECH = GLOBAL.TECH


PrefabFiles = {
	"charliethegardener",
	"tai",
	"taiwings",
	"charliebooks",
}

Assets = {
    Asset( "IMAGE", "images/saveslot_portraits/charliethegardener.tex" ),
    Asset( "ATLAS", "images/saveslot_portraits/charliethegardener.xml" ),

    Asset( "IMAGE", "images/selectscreen_portraits/charliethegardener.tex" ),
    Asset( "ATLAS", "images/selectscreen_portraits/charliethegardener.xml" ),
	
    Asset( "IMAGE", "images/selectscreen_portraits/charliethegardener_silho.tex" ),
    Asset( "ATLAS", "images/selectscreen_portraits/charliethegardener_silho.xml" ),

    Asset( "IMAGE", "bigportraits/charliethegardener.tex" ),
    Asset( "ATLAS", "bigportraits/charliethegardener.xml" ),
	
	Asset( "IMAGE", "images/map_icons/charliethegardener.tex" ),
	Asset( "ATLAS", "images/map_icons/charliethegardener.xml" ),
	
	Asset( "IMAGE", "images/avatars/avatar_charliethegardener.tex" ),
    Asset( "ATLAS", "images/avatars/avatar_charliethegardener.xml" ),
	
	Asset( "IMAGE", "images/avatars/avatar_ghost_charliethegardener.tex" ),
    Asset( "ATLAS", "images/avatars/avatar_ghost_charliethegardener.xml" ),

	Asset( "IMAGE", "images/inventoryimages/book_rain.tex" ),
	Asset ("ATLAS", "images/inventoryimages/book_rain.xml"),

	Asset( "IMAGE", "images/inventoryimages/book_butterflybees.tex" ),
	Asset ("ATLAS", "images/inventoryimages/book_butterflybees.xml"),

	Asset( "IMAGE", "images/inventoryimages/book_charliegardening.tex" ),
	Asset ("ATLAS", "images/inventoryimages/book_charliegardening.xml"),

}

--[[Zackreaver: Making a Modmain change!!
See alot of complicated stuff has to be done to get your character to do things that aren't normally
a feature.  Such as getting your character to gain sanity from planting flowers.
So for a perk like this, we gotta do some adjustments to the flowers directly
However, editing the flower.lua and flower_evil.lua directly would cause conflicts with other mods
so instead, were gonna basically create a logic gate in between the sanity aura of the flowers, and
the actual effect that happens to the characters.  
Basically were gonna find out if the source(flowers) and target(your character) of the sanity aura is what we want.
--]]


local require = GLOBAL.require
local STRINGS = GLOBAL.STRINGS


GLOBAL.ZACK_CHARLIE_CHARACTER_MOD = {
	SpawnTaiForCharlie = function(player)
		player:DoTaskInTime(1,function(player) --Attempt to fix wilderness Tai Spawn
			local taifriend = GLOBAL.SpawnPrefab("tai")
			local x,y,z = player.Transform:GetWorldPosition()
			taifriend.Transform:SetPosition(x,y,z)
			taifriend.AnimState:SetMultColour(0,1,0,1)
			taifriend:LinkToPlayer(player)
		end)
	end,
	TAI_HAPPY_REVIVE = 100,
	TAI_HAPPY_BUTTER = 100, --This is how happy Tai has to be to spawn butter
	TAI_HAPPY = 30, --This is how happy Tai has to be to spawn flowers
	TAI_HAPPY_FLOWER = 2, --This is how much happiness is lost when Tai spawns a flower
	TAI_GIFT_MINSPACING = 3,
	TAI_GIFT_BASETIME = 60,
	TAI_GIFT_RANDTIME = 60,
	TAI_RETRY_GIFT_BASETIME = 3,
	TAI_RETRY_GIFT_RANDTIME = 3,
--	ZACK_Is_ROG_Enabled = function() return SEASONS.AUTUMN end,
}
--First off, only the server should be handling this
if GLOBAL.TheNet:GetIsServer() then

	AddPrefabPostInit("flower", function(inst)--Regular flowers don't have a sanity aura
		inst:AddComponent("sanityaura") --So with this we gave it to them :D
		inst.components.sanityaura.aura = 0 --And with aura set to 0, they don't affect normal players sanity
	end)
	AddComponentPostInit("sanityaura", function(self)
		local old = self.GetAura --Gotta store the old function for GetAura here
		function self:GetAura(observer) --See sanityaura already has a function called GetAura, by putting this here we replaced it
			if observer.prefab == "charliethegardener" and self.inst.prefab == "flower_evil" then --If the source of the aura is an evil flower, and charlie is the players character
				return 0 --Putting this as 0 makes charlie immune to the sanity aura of evil flowers
				--If you raise this evil flowers will heal his sanity, lowering it will make it drain again
			end
			if observer.prefab == "charliethegardener" and self.inst.prefab == "flower" then --If the source of the aura is an flower, and charlie is the players character
				return 0.1 --An evil flower has -0.416666666667 for it's sanity aura, this one is a positive aura of 0.1, a fourth of the bonus in reverse
			end
			if old then
				return old(self,observer) --This means that either the character isn't Charlie, or the aura isn't from an evil flower or regular flower
			end
		end
	end)

	AddPrefabPostInit("flower_evil", function(inst)--Gotta adjust the flowers again
		local old = inst.components.pickable.onpickedfn
		if inst.components.pickable then
			inst.components.pickable.onpickedfn =
				function(inst,picker)
					if picker and picker:HasTag("charliegardener") and picker.components.sanity then
						picker.components.sanity:DoDelta(0) --You can change this 0 to anything you want, negative numbers will drain sanity, positive will increase it
						inst:Remove()
					else
						if old then
							old(inst,picker)
						end
					end
				end
		end
	end)
	
	--Now it's time to make planting things give sanity to Charlie
	--Here's a table that contains all the things charlie gains sanity from planting
	--The number next to it, is how much sanity is gained from it
	local PlantList = {
	butterfly = {sanity = 10, happiness = 20}, 
	dug_grass = {sanity = 5, happiness = 1}, 
	dug_sapling = {sanity = 5, happiness = 1}, 
	dug_berrybush = {sanity = 10, happiness = 5}}
	function charlieondeploy(inst, pt, deployer)
		if deployer and deployer:HasTag("charliegardener") then --If the person planting the thing is charlie
			for k,v in pairs(PlantList) do
				if k == inst.prefab then
					if deployer.components.sanity then
						deployer.components.sanity:DoDelta(v.sanity) --then raise his sanity
					end					
					if deployer.components.leader then
						for k2,v2 in pairs(deployer.components.leader.followers) do
							if k2.prefab == "tai" then
								if k2.taihappiness then
									k2.taihappiness = k2.taihappiness + v.happiness--and tai happiness
								else
									k2.taihappiness = v.happiness--and tai happiness
								end
								if k2.taihappiness >= GLOBAL.ZACK_CHARLIE_CHARACTER_MOD.TAI_HAPPY_BUTTER + GLOBAL.ZACK_CHARLIE_CHARACTER_MOD.TAI_HAPPY then
									k2.components.periodicspawner.prefab = "butter"
								else
									k2.components.periodicspawner.prefab = "flower"								
								end
							end
						end
					end
					if deployer.components.inventory then
						local TaiWings = deployer.components.inventory:FindItem(function (item) return item.prefab == "taiwings" end)
						if TaiWings then
							if TaiWings.taihappiness then
								TaiWings.taihappiness = TaiWings.taihappiness + v.happiness
							else
								TaiWings.taihappiness = v.happiness
							end
							if TaiWings.taihappiness and TaiWings.taihappiness >= GLOBAL.ZACK_CHARLIE_CHARACTER_MOD.TAI_HAPPY_REVIVE then
								TaiWings:Remove()
								GLOBAL.ZACK_CHARLIE_CHARACTER_MOD.SpawnTaiForCharlie(deployer)
								if deployer.components.talker then
									deployer.components.talker:Say("Tai! You came back!")
								end
							end
						end
					end
				end
			end
		end
	end
	function DeployReplacementFunc(inst)
		local old = inst.components.deployable.ondeploy
		if inst.components.deployable then
			inst.components.deployable.ondeploy = 
				function(inst,pt,deployer)
					charlieondeploy(inst,pt,deployer)
					if old then
						old(inst,pt,deployer)
					end
				end
		end
	end
	for k,v in pairs(PlantList) do
		AddPrefabPostInit(k,DeployReplacementFunc)
	end

	
	AddComponentPostInit("beard", function(self)--Gotta nerf dat beard
		local old = self.GetInsulation
		function self:GetInsulation()
			if self.inst:HasTag("charliegardener") then
				return self.bits * 2 --Changing the 15 to a 2 only for charlie.
			end
			if old then
				return old(self)
			end
		end	
	end)
end
--End of host only stuff

--Charlie book time
--local CUSTOM_RECIPETABS = GLOBAL.CUSTOM_RECIPETABS
GLOBAL.RECIPETABS['CHARLIEBOOKS'] = { str = "CHARLIEBOOKS", sort = 69098, icon = "tab_book.tex" }
STRINGS.TABS.CHARLIEBOOKS = "Books" --Custom tabs seem to be broken at the moment
--[[Steampunk Example
GLOBAL.RECIPETABS['GEAR_TAB'] = {str = "GEAR_TAB", sort=1, icon = "gear_tab.tex", icon_atlas = "images/inventoryimages/gear_tab.xml"}
STRINGS.TABS.GEAR_TAB = "Gears"--]]

local butterflybeebook_recipe = AddRecipe("book_butterflybees", {Ingredient("papyrus", 2), Ingredient("bee", 2), Ingredient("butterfly", 2)}, RECIPETABS.CHARLIEBOOKS, {SCIENCE = 2}, nil, nil, nil, nil, "charliebookbuilder")
butterflybeebook_recipe.atlas = GLOBAL.resolvefilepath("images/inventoryimages/book_butterflybees.xml")
STRINGS.RECIPE_DESC.BOOK_BUTTERFLYBEES = "Butterflies and Bees.\nHoney Lovers Edition."
local rainbook_recipe = AddRecipe("book_rain", {Ingredient("papyrus", 2), Ingredient("bluegem", 1), Ingredient("nightmarefuel", 2)}, RECIPETABS.CHARLIEBOOKS, {MAGIC = 3}, nil, nil, nil, nil, "charliebookbuilder")
rainbook_recipe.atlas = GLOBAL.resolvefilepath("images/inventoryimages/book_rain.xml")
STRINGS.RECIPE_DESC.BOOK_RAIN = "Teaches you advanced rain dancing techniques."
local gardening_recipe = AddRecipe("book_charliegardening", {Ingredient("papyrus", 2), Ingredient("seeds", 1), Ingredient("poop", 1)}, RECIPETABS.CHARLIEBOOKS, {SCIENCE = 1}, nil, nil, nil, nil, "charliebookbuilder")
gardening_recipe.atlas = GLOBAL.resolvefilepath("images/inventoryimages/book_charliegardening.xml")
STRINGS.RECIPE_DESC.BOOK_CHARLIEGARDENING = "On the care and feeding of plants."
--[[Steampunk axe Example
local gear_axerecipe = Recipe( "gear_axe", { Ingredient("axe", 1),Ingredient("gears", 1) }, RECIPETABS.GEAR_TAB, TECH.SCIENCE_ONE)
gear_axerecipe.atlas = "images/inventoryimages/gear_axe.xml"--]]


local recipes = 
{
	rainbook_recipe,
	butterflybeebook_recipe,
	gardening_recipe,
}
--#rezecib this is a bit of a hack; hardcoding the sortkeys should avoid mod clashes
-- due to different load order, but if a mod somehow declares sortkeys in this range in
-- the same way, then problems will always occur with that mod
local sortkey = 79098
for k,v in pairs(recipes) do
	sortkey = sortkey - 1
	v.sortkey = sortkey
end
--GLOBAL.STRINGS.RECIPE_DESC.SPIDEREGGSACK = "Get a little help from your friends."--]]


-- The character select screen lines
STRINGS.CHARACTER_TITLES.charliethegardener = "The Gardener"
STRINGS.CHARACTER_NAMES.charliethegardener = "Charlie"
STRINGS.CHARACTER_DESCRIPTIONS.charliethegardener = "*Has a butterfly companion name Tai.\n*Loves nature. *Hates seeing it burn.\n*Had too many Caffeine pills."
STRINGS.CHARACTER_QUOTES.charliethegardener = "\"Go pick flowers from the garden if you start feeling crazy\""

-- Custom speech strings
STRINGS.CHARACTERS.CHARLIETHEGARDENER = require "speech_charliethegardener"

-- The character's name as appears in-game 
STRINGS.NAMES.CHARLIETHEGARDENER = "Charlie"

STRINGS.NAMES.BOOK_BUTTERFLYBEES = "The Butterflies and Bees."
STRINGS.CHARACTERS.CHARLIETHEGARDENER.DESCRIBE.BOOK_BUTTERFLYBEES = "Reading this gets me in the mood for honey."
STRINGS.NAMES.BOOK_RAIN = "Of Rain Dancing and Men."
STRINGS.CHARACTERS.CHARLIETHEGARDENER.DESCRIBE.BOOK_RAIN = "What could possibly go wrong?"
STRINGS.NAMES.BOOK_CHARLIEGARDENING = "Applied Horticulture (Charlie)"
STRINGS.CHARACTERS.CHARLIETHEGARDENER.DESCRIBE.BOOK_CHARLIEGARDENING = "I love gardening!"

STRINGS.NAMES.TAI = "Tai"
STRINGS.NAMES.TAIWINGS = "Tais' Wings"
STRINGS.CHARACTERS.CHARLIETHEGARDENER.DESCRIBE.TAI = "That's Tai"
STRINGS.CHARACTERS.CHARLIETHEGARDENER.DESCRIBE.TAIWINGS = "Tai... I'm sorry."
STRINGS.CHARACTERS.CHARLIETHEGARDENER.DESCRIBE.WAXWELL = "Why does he look so... Disappointed to see me?"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.TAI = "That looks like a very friendly butterfly."
STRINGS.CHARACTERS.GENERIC.DESCRIBE.TAIWINGS = "For some reason, I can't find the heart to eat this."
STRINGS.CHARACTERS.GENERIC.DESCRIBE.BOOK_BUTTERFLYBEE = "Caution: Do not read during spring."
STRINGS.CHARACTERS.GENERIC.DESCRIBE.BOOK_RAIN = "This chapter on lightning strikes worries me."
STRINGS.CHARACTERS.GENERIC.ANNOUNCE_WAYTOOMANYBUGS = "There's way too many bugs here already!"
STRINGS.CHARACTERS.GENERIC.ANNOUNCE_TOOMANYBUGS = "There's too many bugs here already"
STRINGS.CHARACTERS.WICKERBOTTOM.DESCRIBE.BOOK_CHARLIEGARDENING = "Someone wrote all over this book!"


-- The default responses of examining the character
STRINGS.CHARACTERS.GENERIC.DESCRIBE.CHARLIETHEGARDENER = 
{
	GENERIC = "It's Charlie!",
	ATTACKER = "That Charlie looks shifty...",
	MURDERER = "Murderer!",
	REVIVER = "Charlie, friend of ghosts.",
	GHOST = "Charlie could use a heart.",
}


AddMinimapAtlas("images/map_icons/charliethegardener.xml")

-- Add mod character to mod character list. Also specify a gender. Possible genders are MALE, FEMALE, ROBOT, NEUTRAL, and PLURAL.
AddModCharacter("charliethegardener", "MALE")

--These are the debug commands, these should be deleted when the character is completed
--Otherwise alot of cheating can be done 0.0
--[[
GLOBAL.CHEATS_ENABLED = true
GLOBAL.require( 'debugkeys' )
--]]