local assets =
{
	Asset("ANIM", "anim/books.zip"),
	Asset( "IMAGE", "images/inventoryimages/book_rain.tex" ),
	Asset ("ATLAS", "images/inventoryimages/book_rain.xml"),
	Asset( "IMAGE", "images/inventoryimages/book_butterflybees.tex" ),
	Asset ("ATLAS", "images/inventoryimages/book_butterflybees.xml"),
	Asset( "IMAGE", "images/inventoryimages/book_charliegardening.tex" ),
	Asset ("ATLAS", "images/inventoryimages/book_charliegardening.xml"),
}
local prefabs =
{

}
function PickBug()
	local bug = {"butterfly","bee"}
	return bug[math.random(1,2)]
end

function SpawnBug(spawnpoint)
    local prefab = PickBug()
    if prefab == nil then
        return
    end

    local bug = SpawnPrefab(prefab)
    if math.random() < .5 then
        bug.Transform:SetRotation(180)
    end

    bug.Physics:Teleport(spawnpoint:Get())

    return bug
end


function butterflybeesfn(inst, reader)
    local birdspawner = TheWorld.components.birdspawner
    if birdspawner == nil then
        return false
    end

    local pt = Vector3(reader.Transform:GetWorldPosition())
    local num = 20 + math.random(10)

    reader.components.sanity:DoDelta(-TUNING.SANITY_HUGE)
    
    --we can actually run out of command buffer memory if we allow for infinite birds
	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, 10, nil, nil, { "magicalbug" })
	if #ents > 30 then
		num = 0
		reader.components.talker:Say(GetString(reader.prefab, "ANNOUNCE_WAYTOOMANYBUGS"))
	elseif #ents > 20 then
		reader.components.talker:Say(GetString(reader.prefab, "ANNOUNCE_TOOMANYBUGS"))
		num = 10 + math.random(10)
	end
    
    if num > 0 then
		reader:StartThread(function()
			for k = 1, num do
				local pos = birdspawner:GetSpawnPoint(pt) --This still works for finding proper ground
				if pos ~= nil then
					local bug = SpawnBug(pos)
					SpawnPrefab("small_puff").Transform:SetPosition(pos:Get())
                    if bug ~= nil then
					   bug:AddTag("magicalbug")
                    end
				end
				Sleep(math.random(.2, .25))
			end
		end)
	end
    return true
end

function rainfn(inst, reader)
    local pt = Vector3(reader.Transform:GetWorldPosition())

    reader.components.sanity:DoDelta(-TUNING.SANITY_LARGE)

    local range = 90
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, range)
    for k, v in pairs(ents) do
		if v.components.burnable then
			v.components.burnable:Extinguish()
		end
	end
	TheWorld:PushEvent("ms_forceprecipitation")
    return true
end

function growfn(inst, reader)
    local pt = Vector3(reader.Transform:GetWorldPosition())

    reader.components.sanity:DoDelta(-TUNING.SANITY_LARGE)

    local range = 30
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, range)
    for k, v in pairs(ents) do
        if v.components.pickable ~= nil then
            v.components.pickable:FinishGrowing()
        end

        if v.components.crop ~= nil then
            v.components.crop:DoGrow(TUNING.TOTAL_DAY_TIME * 3)
        end
        
        if v.components.growable ~= nil and v:HasTag("tree") and not v:HasTag("stump") then
            v.components.growable:DoGrowth()
        end
    end
    return true
end

function MakeBook(name, usefn, bookuses)
	local function fn()
    	local inst = CreateEntity()

    	inst.entity:AddTransform()
    	inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()
        inst.entity:AddNetwork()

        MakeInventoryPhysics(inst)

        inst.AnimState:SetBank("books")
        inst.AnimState:SetBuild("books")
		if name == "book_rain" then
			inst.AnimState:PlayAnimation("book_sleep")
		else
			inst.AnimState:PlayAnimation("book_brimstone")
		end
		if name == "book_charliegardening" then
			inst.AnimState:PlayAnimation("book_gardening")
		end
        if not TheWorld.ismastersim then
            return inst
        end

        inst.entity:SetPristine()

        -----------------------------------
        
        inst:AddComponent("inspectable")
        inst:AddComponent("book")
        inst.components.book.onread = usefn

        inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.imagename = ""..name
		inst.components.inventoryitem.atlasname = "images/inventoryimages/"..name..".xml"


        inst:AddComponent("finiteuses")
        inst.components.finiteuses:SetMaxUses(bookuses)
        inst.components.finiteuses:SetUses(bookuses)
        inst.components.finiteuses:SetOnFinished(inst.Remove)

        MakeSmallBurnable(inst)
        MakeSmallPropagator(inst)

        MakeHauntableLaunch(inst)
        AddHauntableCustomReaction(inst, function(inst, haunter)
            if math.random() <= TUNING.HAUNT_CHANCE_OCCASIONAL then
                inst.components.book.onread(inst, haunter)
                inst.components.hauntable.hauntvalue = TUNING.HAUNT_SMALL
                return true
            end
            return false
        end, true, false, true)

        return inst
    end
    return Prefab("common/inventory/"..name, fn, assets, prefabs)
end


return MakeBook("book_rain", rainfn, 3),
       MakeBook("book_butterflybees", butterflybeesfn, 5), 
	   MakeBook("book_charliegardening", growfn, 5)
