	--[[Zackreaver: Hello friend, it's your little scripting buddy, I'm going to introduce 
	you to the inner workings of your character, Take note that when I put a --[ [ and --] ] into
	the script, it will turn everything between them into comments.
	
	This is great for leaving you rather large messages like this, or just outright shutting off large
	sections of script without actually deleting it.
	
	Notepad++ also lets you click a little minus button on the left side that lets you minimize these sections
	Keeping tidy while scripting is important (Otherwise we get lost quick 0.0)
	
	At any rate, if you wanna know how your character works lets start at the beginning.  Head down to the
	local master_postinit = function(inst) part.
	--]]

local MakePlayerCharacter = require "prefabs/player_common"

local assets = {

        Asset( "ANIM", "anim/player_basic.zip" ),
        Asset( "ANIM", "anim/player_idles_shiver.zip" ),
        Asset( "ANIM", "anim/player_actions.zip" ),
        Asset( "ANIM", "anim/player_actions_axe.zip" ),
        Asset( "ANIM", "anim/player_actions_pickaxe.zip" ),
        Asset( "ANIM", "anim/player_actions_shovel.zip" ),
        Asset( "ANIM", "anim/player_actions_blowdart.zip" ),
        Asset( "ANIM", "anim/player_actions_eat.zip" ),
        Asset( "ANIM", "anim/player_actions_item.zip" ),
        Asset( "ANIM", "anim/player_actions_uniqueitem.zip" ),
        Asset( "ANIM", "anim/player_actions_bugnet.zip" ),
        Asset( "ANIM", "anim/player_actions_fishing.zip" ),
        Asset( "ANIM", "anim/player_actions_boomerang.zip" ),
        Asset( "ANIM", "anim/player_bush_hat.zip" ),
        Asset( "ANIM", "anim/player_attacks.zip" ),
        Asset( "ANIM", "anim/player_idles.zip" ),
        Asset( "ANIM", "anim/player_rebirth.zip" ),
        Asset( "ANIM", "anim/player_jump.zip" ),
        Asset( "ANIM", "anim/player_amulet_resurrect.zip" ),
        Asset( "ANIM", "anim/player_teleport.zip" ),
        Asset( "ANIM", "anim/wilson_fx.zip" ),
        Asset( "ANIM", "anim/player_one_man_band.zip" ),
        Asset( "ANIM", "anim/shadow_hands.zip" ),
        Asset( "SOUND", "sound/sfx.fsb" ),
        Asset( "SOUND", "sound/wilson.fsb" ),
        Asset( "ANIM", "anim/beard.zip" ),

        Asset( "ANIM", "anim/charliebeard.zip" ),
        Asset( "ANIM", "anim/charliethegardener.zip" ),
        Asset( "ANIM", "anim/ghost_charliethegardener_build.zip" ),
}
local prefabs = 
{
	"beardhair",
	"taiwings",
}
local start_inv = {
	-- Custom starting items
	"bugnet",
}

local function OnResetBeard(inst)
    inst.AnimState:ClearOverrideSymbol("beard")
end

--tune the beard economy...
local BEARD_DAYS = { 2, 4, 8 }
local BEARD_BITS = { 3, 9, 40 }

local function OnGrowShortBeard(inst)
    inst.AnimState:OverrideSymbol("beard", "charliebeard", "beard_short")
    inst.components.beard.bits = BEARD_BITS[1]
end

local function OnGrowMediumBeard(inst)
   inst.AnimState:OverrideSymbol("beard", "charliebeard", "beard_medium")
    inst.components.beard.bits = BEARD_BITS[2]
end

local function OnGrowLongBeard(inst)
    inst.AnimState:OverrideSymbol("beard", "charliebeard", "beard_long")
    inst.components.beard.bits = BEARD_BITS[3]
end

local NatureList = 
{
--This is a table of most objects of nature along with their sanity drain value while burning
--The higher the number on the right, the more sanity that gets drained.
	evergreen = 1,
	evergreen_sparse = 2,
	flower = 1.5,
	beehive = 1.5,
	flower_evil = 1,
	sapling = 2,
	grass = 2,
	berrybush = 2,
	red_mushroom = 2,
	green_mushroom = 2,
	blue_mushroom = 2
}

function IsNature(natureprefab)
	for k,v in pairs(NatureList) do
		if k == natureprefab then
			return v
		end
	end
	return false
end

local function sanityfn(inst)
	local x,y,z = inst.Transform:GetWorldPosition()	
	local delta = 0
	local max_rad = 40 --The max radius that will be checked for objects in nature which are burning
	--Nature burning outside of the max radius will not lower Charlie's sanity
	local stretch = 10 --The higher this number, the farther the radius will be stretched out
	--Lowering the stretch will make the sanity aura have a smaller radius
	local ents = TheSim:FindEntities(x,y,z, max_rad, {"fire"})
    for k,v in pairs(ents) do
    	if v.components.burnable and v.components.burnable.burning and IsNature(v.prefab) then
            local distsq = inst:GetDistanceSqToInst(v)
    		local sz = -IsNature(v.prefab)/math.max(1, distsq / stretch)

            delta = delta + sz
    	end
    end

    return delta
end


local function OnDespawn(inst)
    if inst.taifriend ~= nil and inst.taifriend.components.health and inst.taifriend.components.health:IsDead() == false then
        inst.taifriend.components.lootdropper:SetLoot(nil)
        inst.taifriend.components.health:SetInvincible(true)
		inst.taifriend.sg:GoToState("land")
        inst.taifriend:DoTaskInTime(2, inst.taifriend.Remove)
    end
end

local function OnSave(inst, data)
    if inst.taifriend ~= nil and inst.taifriend.components.health and inst.taifriend.components.health:IsDead() == false then
        data.taifriend = inst.taifriend:GetSaveRecord()
    end
	if inst.initialtaispawn then
		data.initialtaispawn = inst.initialtaispawn
	end--
end

local function OnLoad(inst, data)
    if data.taifriend ~= nil and inst.taifriend == nil then
        local taifriend = SpawnSaveRecord(data.taifriend)
        if taifriend ~= nil then
--            taifriend.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn") --I might spawn a sound in the future
			taifriend.AnimState:SetMultColour(0,1,0,1)
			taifriend:LinkToPlayer(inst)
        end
	end
	if data.initialtaispawn then
		inst.initialtaispawn = data.initialtaispawn
	end--
end

--[[function SpawnTaiForCharlie(player) --Out of date, I have a global table on ModMain that handles this
		local taifriend = SpawnPrefab("tai")
		local x,y,z = player.Transform:GetWorldPosition()
		taifriend.Transform:SetPosition(x,y,z)
		taifriend.AnimState:SetMultColour(0,1,0,1)
		taifriend:LinkToPlayer(player)
	end--]]


-- This initializes for both clients and the host
local common_postinit = function(inst) 
	-- Minimap icon
    inst:AddTag("insomniac") --This is the tag that prevents wickerbottom from sleeping
    inst:AddTag("charliebookbuilder") --This tag lets Charlie build books
	inst.MiniMapEntity:SetIcon( "charliethegardener.tex" )
end



-- This initializes for the host only
local master_postinit = function(inst)
	--[[Hey there again, this is the part where most of the work is done by your mod.
	Understand that in don't starve together everything is handled by the host.
	Whenever a client (someone who's not the host) does something, they are just telling the host
	to do it for them, and thats how networking works.
	
	But some things (like screen graphics at low sanity) are good to do on client side as well
	First, lets go over the things that come with the template
	--]]
	-- choose which sounds this character will play 
	--Zackreaver: You can change the "wilson" to anything from "willow" to "wickerbottom" etc
	--You can even make your own custom sounds and put them in, so theres that too.
	inst.soundsname = "wickerbottom"
	-- Stats	--Zackreaver: You can adjust your characters starting stats right here.
	inst.components.health:SetMaxHealth(150)
	inst.components.hunger:SetMax(150)
	inst.components.sanity:SetMax(200)
    inst:AddTag("charliegardener") --This tag is better than relying on prefab checks
	
	--Everywhere from this point has been added in by me, and I'll explain what it does.
	inst:AddComponent("beard") --This is wilson's beard component, the stuff below it controls the details of it.
    inst.components.beard.onreset = OnResetBeard
    inst.components.beard.prize = "cutgrass"
    inst.components.beard:AddCallback(BEARD_DAYS[1], OnGrowShortBeard)
    inst.components.beard:AddCallback(BEARD_DAYS[2], OnGrowMediumBeard)
    inst.components.beard:AddCallback(BEARD_DAYS[3], OnGrowLongBeard)
	
	--Lets make your character lose sanity when nature is on fire
	inst.components.sanity.custom_rate_fn = sanityfn
	
	--Lets make charlie able to read books
	inst:AddComponent("reader")
	
	--Knows these recipes from the get go
	inst.components.builder:AddRecipe("backpack")
	inst.components.builder:AddRecipe("bugnet")
	inst.components.builder:AddRecipe("shovel")
		
	--When a Charlie is chosen, spawn Tai and make charlie his friend.
--
	local SpawnTaifn = function(world, playerspawntable)
			local player = playerspawntable--.player
			if player then
				player:DoTaskInTime(1,function(player)
					if player:HasTag("charliegardener") and player.initialtaispawn ~= true then
						ZACK_CHARLIE_CHARACTER_MOD.SpawnTaiForCharlie(player)
						player.initialtaispawn = true --Need to prevent duplicate Tai's
					end
				end)
			end
		end
	if TheWorld:HasTag("taispawnerlistener") ~= true then
--		TheWorld:ListenForEvent("ms_newplayercharacterspawned", SpawnTaifn) --Tai doesn't spawn on wilderness with this
		TheWorld:ListenForEvent("ms_playerjoined", SpawnTaifn) 
		TheWorld:AddTag("taispawnerlistener")
	end--
	
	
	inst.OnDespawn = OnDespawn
    inst.OnSave = OnSave
    inst.OnLoad = OnLoad
--	inst.OnNewSpawn = ZACK_CHARLIE_CHARACTER_MOD.SpawnTaiForCharlie --I should see if this works

	--That's all that we can do here, but this isn't all that controls your character
	--Head over to the modmain.lua and look for for my name again
end

return MakePlayerCharacter("charliethegardener", prefabs, assets, common_postinit, master_postinit, start_inv)
